# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V38
12294
12296
12321
12330
12344
12347
13312
19904
19968
40957
63744
64110
64112
64218
94180
94181
94208
100344
100352
101590
101632
101641
110960
111356
131072
173790
173824
177973
177984
178206
178208
183970
183984
191457
194560
195102
196608
201547
END
