# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V30
3656
3660
3784
3788
4151
4152
4195
4197
4201
4206
4231
4238
4239
4240
4250
4252
6600
6602
6773
6778
43307
43310
43643
43646
43711
43712
43713
43714
44012
44013
END
