+{
   locale_version => 1.27,
   entry => <<'ENTRY', # for DUCET v10.0.0
0103      ; [.1CAE.0020.0002] # LATIN SMALL LETTER A WITH BREVE
0061 0306 ; [.1CAE.0020.0002] # LATIN SMALL LETTER A WITH BREVE
0102      ; [.1CAE.0020.0008] # LATIN CAPITAL LETTER A WITH BREVE
0041 0306 ; [.1CAE.0020.0008] # LATIN CAPITAL LETTER A WITH BREVE
1EB1      ; [.1CAE.0020.0002][.0000.0025.0002] # LATIN SMALL LETTER A WITH BREVE AND GRAVE
1EB0      ; [.1CAE.0020.0008][.0000.0025.0002] # LATIN CAPITAL LETTER A WITH BREVE AND GRAVE
1EAF      ; [.1CAE.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER A WITH BREVE AND ACUTE
1EAE      ; [.1CAE.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER A WITH BREVE AND ACUTE
1EB5      ; [.1CAE.0020.0002][.0000.002D.0002] # LATIN SMALL LETTER A WITH BREVE AND TILDE
1EB4      ; [.1CAE.0020.0008][.0000.002D.0002] # LATIN CAPITAL LETTER A WITH BREVE AND TILDE
1EB3      ; [.1CAE.0020.0002][.0000.003B.0002] # LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE
1EB2      ; [.1CAE.0020.0008][.0000.003B.0002] # LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE
1EB7      ; [.1CAE.0020.0002][.0000.0042.0002] # LATIN SMALL LETTER A WITH BREVE AND DOT BELOW
1EB6      ; [.1CAE.0020.0008][.0000.0042.0002] # LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW
00E2      ; [.1CAF.0020.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX
0061 0302 ; [.1CAF.0020.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX
00C2      ; [.1CAF.0020.0008] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
0041 0302 ; [.1CAF.0020.0008] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
1EA7      ; [.1CAF.0020.0002][.0000.0025.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE
1EA6      ; [.1CAF.0020.0008][.0000.0025.0002] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE
1EA5      ; [.1CAF.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE
1EA4      ; [.1CAF.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE
1EAB      ; [.1CAF.0020.0002][.0000.002D.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE
1EAA      ; [.1CAF.0020.0008][.0000.002D.0002] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE
1EA9      ; [.1CAF.0020.0002][.0000.003B.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
1EA8      ; [.1CAF.0020.0008][.0000.003B.0002] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
1EAD      ; [.1CAF.0020.0002][.0000.0042.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW
1EAC      ; [.1CAF.0020.0008][.0000.0042.0002] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW
00EE      ; [.1D99.0020.0002] # LATIN SMALL LETTER I WITH CIRCUMFLEX
0069 0302 ; [.1D99.0020.0002] # LATIN SMALL LETTER I WITH CIRCUMFLEX
00CE      ; [.1D99.0020.0008] # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
0049 0302 ; [.1D99.0020.0008] # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
015F      ; [.1ED8.0020.0002] # LATIN SMALL LETTER S WITH CEDILLA
0073 0327 ; [.1ED8.0020.0002] # LATIN SMALL LETTER S WITH CEDILLA
0219      ; [.1ED8.0020.0002] # LATIN SMALL LETTER S WITH COMMA BELOW
0073 0326 ; [.1ED8.0020.0002] # LATIN SMALL LETTER S WITH COMMA BELOW
015E      ; [.1ED8.0020.0008] # LATIN CAPITAL LETTER S WITH CEDILLA
0053 0327 ; [.1ED8.0020.0008] # LATIN CAPITAL LETTER S WITH CEDILLA
0218      ; [.1ED8.0020.0008] # LATIN CAPITAL LETTER S WITH COMMA BELOW
0053 0326 ; [.1ED8.0020.0008] # LATIN CAPITAL LETTER S WITH COMMA BELOW
0163      ; [.1EFC.0020.0002] # LATIN SMALL LETTER T WITH CEDILLA
0074 0327 ; [.1EFC.0020.0002] # LATIN SMALL LETTER T WITH CEDILLA
021B      ; [.1EFC.0020.0002] # LATIN SMALL LETTER T WITH COMMA BELOW
0074 0326 ; [.1EFC.0020.0002] # LATIN SMALL LETTER T WITH COMMA BELOW
0162      ; [.1EFC.0020.0008] # LATIN CAPITAL LETTER T WITH CEDILLA
0054 0327 ; [.1EFC.0020.0008] # LATIN CAPITAL LETTER T WITH CEDILLA
021A      ; [.1EFC.0020.0008] # LATIN CAPITAL LETTER T WITH COMMA BELOW
0054 0326 ; [.1EFC.0020.0008] # LATIN CAPITAL LETTER T WITH COMMA BELOW
ENTRY
};
